/*
 * Copyright 2024 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef XUNFEIVISIONENGINE_P_H
#define XUNFEIVISIONENGINE_P_H

#include <functional>
#include <string>
#include <kylin-ai/ai-engine/large-model/aiengineerror.h>
#include <kylin-ai/ai-engine/large-model/cloudvisionengine.h>
#include <cpr/cpr.h>

class XunfeiVisionEnginePrivate : public ai_engine::lm::vision::AbstractCloudVisionEngine {
public:
    XunfeiVisionEnginePrivate();
    ~XunfeiVisionEnginePrivate();
    std::string engineName();

    void setConfig(const std::string &config) override;

    void setPrompt2ImageCallback(ai_engine::lm::vision::ImageResultCallback callback) override;
    void setPrompt2ImageSize(int width, int height) override;
    void setPrompt2ImageNumber(int number) override;
    void setPrompt2ImageStyle(int style) override;
    std::string getPrompt2ImageSupportedParams() const override;

    bool prompt2Image(const std::string &prompt, ai_engine::lm::EngineError &error) override;

private:
    std::string appId_;
    std::string apiKey_;
    std::string secretKey_;
    std::string hostUrl_{"https://spark-api.cn-huabei-1.xf-yun.com/v2.1/tti"};

    ai_engine::lm::vision::ImageResultCallback imageResultCallback_;

    friend bool writePrompt2ImageResult(std::string data, intptr_t userData);
    bool processPrompt2ImageResult(cpr::Response &response, ai_engine::lm::EngineError &error);

    int imageWidth_{1024};
    int imageHeight_{1024};
    int imageNumber_{1};
    std::string imageStyle_;

    std::string incompleteResult_;

    ai_engine::lm::EngineError currentError_;
    bool currentStatus();
    void runCallbackWithError(const ai_engine::lm::EngineError &error, const int &index, const int &total);
};

#endif // XUNFEIVISIONENGINE_P_H
